<?php

namespace Frenzy\Helpers;

class HelperFunctions {

	/**
	 * Clean a url to account for discrepancies coming from API.
	 *
	 * @param string $url The url to clean.
	 *
	 * @return string
	 */
	public static function clean_url( $url = '' ) {

		$url_parts = parse_url( $url );

		if ( empty( $url_parts['host'] ) && ! empty( $url_parts['scheme'] ) ) {
			$path = trim( $url_parts['path'], '/' );
			$url  = $url_parts['scheme'] . '://' . $path;
		}

		return $url;
	}

	public static function get_frenzy_shop_hover_or_always() {
		$allowed     = [ 1, '', null ];
		$saved_value = get_option( 'fetch-shop-btn-show-always', true );

		if ( in_array( $saved_value, $allowed ) ) {
			$show_always = 1;
		} else {
			$show_always = 0;
		}

		return $show_always;
	}

	/**
	 * @return bool True if the plugin is enabled on this post.
	 * @access private
	 */
	public static function frenzy_is_enabled( $post_id ) {
		if ( empty( $post_id ) ) {
			global $post;
			$post_id = ( ! empty( $post->ID ) ? $post->ID : '' );
		}

		return apply_filters( 'frenzy_is_enabled', true, $post_id );
	}

	public static function get_shopstyle_api_param_map() {
		return [
			'categories' => 'cat=%s',
			'brands'     => 'fl=b%d',
			'retailers'  => 'fl=r%d',
			'colors'     => 'fl=c%d',
			'prices'     => 'fl=p%s',
		];
	}

	public static function get_price_filter_options() {
		return [
			'0-50'     => '$50 &amp; under',
			'50-100'   => '$50-$100',
			'100-200'  => '$100-200',
			'200-300'  => '$200-$300',
			'300-400'  => '$300-$400',
			'400-500'  => '$400-$500',
			'500-9999' => '$500+',
		];
	}

	/**
	 * Check if ShopStyle API key is entered in settings.
	 *
	 * @return bool
	 */
	public static function frenzy_has_shopstyle_api_key() {
		$shopstyleApiKey = get_option( 'shopstyle-api-key' );

		return ! empty( $shopstyleApiKey );
	}

	public static function get_fetch_api_key() {
		$account = get_option( 'fetch-account', [ 'apiKey' => '' ] );

		return $account['apiKey'];
	}

	public static function is_fetch_logged_in() {
		$apiKey    = self::get_fetch_api_key();
		$logged_in = false;
		if ( ! empty( $apiKey ) ) {
			$logged_in = true;
		}

		return $logged_in;
	}

	public static function get_logged_in_fetch_account_info() {
		return get_option( 'fetch-account' );
	}

	public static function save_fetch_account_info( $account_details ) {
		update_option(
			'fetch-account',
			[
				'apiKey' => $account_details['apiKey'],
				'name'   => $account_details['name'],
				'email'  => $account_details['email'],
				'indi'   => $account_details['indi'],
			],
			false
		);
	}

	public static function fetch_logout() {
		delete_option( 'fetch-account' );
	}

	public static function get_shop_button_vertical_location() {
		return get_option( 'fetch-position-vertical', 'top' );
	}

	public static function get_shop_button_horizontal_location() {
		return get_option( 'fetch-position-horizontal', 'left' );
	}

	public static function get_show_on_homepage() {
		return get_option( 'fetch-show-home-feed', false );
	}

	/**
	 * Get meta value from fetch_show_shop_button, if exists, and add to fetch_post_settings, then delete fetch_show_shop_button row.
	 *
	 * @param int $post_id Current post.
	 */
	public static function _deprecated_get_fetch_shop_button_setting( $post_id ){

		$show_shop_button = 1;
		if( metadata_exists( 'post', $post_id, 'fetch_show_shop_button' ) ) {
			$show_shop_button = get_post_meta( $post_id, 'fetch_show_shop_button', true );
			delete_post_meta( $post_id, 'fetch_show_shop_button' );

			$updated_post_settings = array(
				'showShopButton' => $show_shop_button,
				'showPrice'      => 1,
			);
			update_post_meta( $post_id, 'fetch_post_settings', $updated_post_settings );

		}

		return $show_shop_button;
	}

	/**
	 * Get fetch post settings.
	 *
	 * @param int $post_id Current post.
	 */
	public static function get_fetch_shop_settings( $post_id ){

		$show_shop_button    = self::_deprecated_get_fetch_shop_button_setting( $post_id );
		$fetch_post_settings = get_post_meta( $post_id, 'fetch_post_settings', true );

		if( ! is_array( $fetch_post_settings ) ) {
			$fetch_post_settings = array(
				'showShopButton' => $show_shop_button,
				'showPrice'      => 1,
			);
		}

		return $fetch_post_settings;

	}

	public static function get_paypal_email() {
		return get_option( 'fetch-paypal-email' );
	}
}
