<?php

namespace Frenzy\Inc;

use Frenzy\Helpers\HelperFunctions;

class FetchShortcode {
	/**
	 * Counter for carousel instances
	 *
	 * @var bool
	 */
	public static $count = 0;

	public function init() {
		add_shortcode( 'fetch', [ $this, 'fetch_product_carousel' ] );
	}

	public function fetch_product_carousel( $atts ) {
	
		$atts = shortcode_atts(
			[
				'ids'   => [],
				'title' => 'Featured Products',
				'type'  => 'carousel',
				'price_toggle'  => 'yes',
			],
			$atts,
			'fetch'
		);

		
		$products = [];
		if ( ! empty( $atts['ids'] ) ) {
			$ids      = explode( ',', str_replace( ' ', '', $atts['ids'] ) );
			$products = get_terms(
				[
					'taxonomy'   => [
						'fetch-product',
					],
					'include'    => $ids,
					'hide_empty' => false,
					'orderby'    => 'include',
				]
			);
		}

		if ( ! is_array( $products ) || empty( $products ) ) {
			$atts['type'] = 'admin-added';
			if ( ! is_admin() && ! defined( 'REST_REQUEST' ) ) {
				return;
			}
		}

		// Default to carousel.
		if ( ! in_array( $atts['type'], [ 'carousel', 'columns-2', 'columns-3', 'columns-4', 'admin-added' ] ) ) {
			$atts['type'] = 'carousel';
		}

		$html = '';
		switch ( $atts['type'] ) {
			case 'admin-added':
				$html = '<div>Click to add products to this carousel. If no products are added, nothing will show on the front end of the site.</div>';
				break;
			case 'carousel':
				self::$count += 1;

				$html  = '<div class="frenzy-product-carousel" id="frenzy-product-carousel-' . self::$count . '" data-product-count="' . count( $products ) . '">';
				$html .= '<div id="frenzy-navigate-left-' . self::$count . '" class="frenzy-carousel-navigation frenzy-navigate-left" ></div>';
				$html .= '<div id="frenzy-navigate-right-' . self::$count . '" class="frenzy-carousel-navigation frenzy-navigate-right" ></div>';

				$html .= '<div class="frenzy-content-wrapper" id="frenzy-content-wrapper-' . self::$count . '">';
				$html .= '<div class="frenzy-content" id="frenzy-content-' . self::$count . '">';
				$html .= '<ul class="frenzy-image-list" id="frenzy-carousel-images-' . self::$count . '">';
				foreach ( $products as $product ) {
					$product_meta = get_term_meta( $product->term_id, 'fetch_product_data', true );

					$html .= '<li data-id="' . esc_attr( $product_meta['ItemID'] ) . '">';
					$html .= '<a href="' . $product_meta['Source'] . '" target="_blank">';
					$html .= '<div class="product-inner">';
					$html .= '<div class="fetch-single-image">';
					if ( ! empty( $product_meta['ImageUrls'][0] ) ) {
						$html .= '<img src="' . esc_url( $product_meta['ImageUrls'][0] ) . '" class="fetch-product-carousel-image">';
					} else {
						$html .= '<span class=""></span>';
					}
					$html .= '</div>';
					$html .= '<div class="fetch-brand">' . esc_html( $product_meta['Brand'] ) . '</div>';
					$html .= '<div class="fetch-item-title">' . esc_html( $product_meta['Title'] ) . '</div>';
					if( $this->should_show_price() ) {
						$html .= '<div class="fetch-price" data-sale="' . esc_attr( (bool) $product_meta['Onsale'] ) . '">$' . esc_html( $product_meta['Price'] ) . '</div>';
					}
					$html .= '</div>';
					$html .= '</a>';
					$html .= '</li>';
				}
				$html .= '</ul>';
				$html .= '</div>';
				$html .= '</div>';
				$html .= '</div>';
				break;
			default:
				$html      = '<ul class="products clearfix fetch-product-grid ' . $atts['type'] . '">';
				$prodcount = 1;
				foreach ( $products as $product ) {
					$product_meta = get_term_meta( $product->term_id, 'fetch_product_data', true );
					$class        = '';
					$html        .= '<li class="product ' . $class . '" data-id="' . esc_attr( $product_meta['ItemID'] ) . '">';
					$html        .= '<a href="' . $product_meta['Source'] . '" target="_blank">';
					$html        .= '<div class="product-inner">';
					$html        .= '<div class="fetch-image-container">';
					if ( ! empty( $product_meta['ImageUrls'][0] ) ) {
						$html .= '<img src="' . esc_url( $product_meta['ImageUrls'][0] ) . '" class="fetch-product-grid-image">';
					} else {
						$html .= '<span class="fetch-clothing"></span>';
					}
					$html .= '</div>';
					$html .= '<div class="fetch-brand">' . esc_html( $product_meta['Brand'] ) . '</div>';
					$html .= '<div class="fetch-item-title">' . esc_html( $product_meta['Title'] ) . '</div>';
					if( $this->should_show_price() ) {
						$html .= '<div class="fetch-price" data-sale="' . esc_attr( (bool) $product_meta['Onsale'] ) . '">$' . esc_html( $product_meta['Price'] ) . '</div>';
					}
					$html .= '</div>';
					$html .= '</a>';
					$html .= '</li>';
					$prodcount ++;
				}
				$html .= '</ul>';
				break;
		}

		return $html;
	}

	function should_show_price( ) {
		$fetch_settings = HelperFunctions::get_fetch_shop_settings( get_the_ID() );
		return $fetch_settings['showPrice'] > 0;
	}
}
