<?php
/**
 * @category  Controller
 * @package   Frenzy
 * @copyright 2018 TODO
 * @license   http://www.gnu.org/licenses/gpl-3.0.txt GPL-3.0+
 * @link      https://frenzy.ai
 */

namespace Frenzy;

use Ari\Database\Helper;
use DOMDocument;
use Frenzy\API\FrenzyClient;
use Frenzy\API\ShopStyleClient;
use Frenzy\Cache\FrenzyCache;
use Frenzy\Helpers\HelperFunctions;
use Frenzy\Inc\FetchGutenberg;
use Frenzy\Inc\FetchShortcode;
use Frenzy\Inc\FetchTaxonomyFetchProduct;
use Frenzy\PluginAPI\Manager;
use Frenzy\Subscriber\AdminStyleAssetSubscriber;
use Frenzy\Subscriber\AjaxSubscriber;
use Frenzy\Subscriber\PublicScriptAssetSubscriber;
use Frenzy\Subscriber\PublicStyleAssetSubscriber;
use Frenzy\Subscriber\SavePaypalSubscriber;
use Frenzy\Subscriber\SaveSettingsSubscriber;
use Frenzy\Subscriber\ScriptAssetSubscriber;
use Frenzy\Subscriber\ShopStyleAjaxSubscriber;
use Frenzy\Subscriber\ShopWidgetSubscriber;
use Frenzy\Subscriber\ToolsMenuSubscriber;

/**
 * Registers all of the subscribers responsible for powering this plugin.
 *
 * @category  Controller
 * @package   Frenzy
 * @author    TODO <todo@todo.com>
 * @copyright 2018 TODO
 * @license   http://www.gnu.org/licenses/gpl-3.0.txt GPL-3.0+
 * @link      https://frenzy.ai
 */
class Plugin {
	/**
	 * Absolute path to the directory where WordPress installed the plugin.
	 *
	 * @var string
	 */
	protected $pluginPath;
	/**
	 * URL to the directory where WordPress installed the plugin.
	 *
	 * @var string
	 */
	protected $pluginUrl;
	/**
	 * The plugin event manager.
	 *
	 * @var Manager
	 */
	private $pluginApiManager;
	/**
	 * The plugin cache.
	 *
	 * @var FrenzyCache
	 */
	private $cache;

	/**
	 * Flag to track if the plugin is loaded.
	 *
	 * @var bool
	 */
	private $loaded;

	/**
	 * Abstract_Plugin constructor.
	 *
	 * @param string $file file path used to determine plugin path and plugin url.
	 */
	public function __construct( $file ) {
		$this->pluginApiManager = new Manager();
		$this->loaded           = false;
		$this->pluginPath       = plugin_dir_path( $file );
		$this->pluginUrl        = plugin_dir_url( $file );
		$this->cache            = new FrenzyCache();
		add_action( 'wp_ajax_fetch-get-preview-data', [ $this, 'wp_ajax_fetch_get_preview_data' ] );
		if ( ! is_admin() ) {
			add_filter( 'pre_get_posts', [ $this, 'frenzy_preview' ] );
			add_filter( 'query_vars', [ $this, 'query_vars' ] );
		}
		add_action( 'admin_footer', [ $this, 'edit_med' ], 999 );

		add_action( 'wp_ajax_frenzy_save_image_products', [ $this, 'frenzy_me_ajax_save_image_products' ] );
		add_action( 'wp_ajax_frenzy_sort_order', [ $this, 'frenzy_ajax_save_sort_order' ] );

		add_action( 'print_media_templates', [ $this, 'wpse_print_media_templates' ] );
		add_action( 'wp_ajax_query-attachments', [ $this, 'frenzy_me_ajax_query_attachments' ], 1 );
		add_action( 'wp_ajax_query_fetch_products', [ $this, 'frenzy_me_ajax_query_attachments' ], 1 );
		add_action( 'wp_ajax_query-attachments', [ $this, 'frenzy_ajax_modify_attachments' ], 1 );
		add_filter( 'wp_prepare_attachment_for_js', [ $this, 'frenzy_add_attachment_products' ], 50, 3 );
		add_filter( 'wp_ajax_frenzy_get_product_info', [ $this, 'frenzy_get_product_info' ] );
		add_filter( 'wp_ajax_frenzy_remove_product_from_image', [ $this, 'frenzy_remove_product_from_image' ] );
		add_filter( 'wp_ajax_fetch_save_draft_post_content', [ $this, 'fetch_save_draft_post_content' ] );
		add_filter( 'wp_ajax_fetch_save_shop_settings', [ $this, 'ajax_fetch_save_shop_settings' ] );
		add_filter( 'wp_get_attachment_image_src', [ $this, 'maybe_update_image_preview_url' ], 10, 4 );
		add_filter( 'wp_get_attachment_url', [ $this, 'maybe_update_image_url' ], 10, 2 );

		add_action( 'wp_enqueue_scripts', [ $this, 'product_carousel_preview_styles' ] );
	}

	public function maybe_update_image_preview_url( $image, $attachment_id, $size, $icon ) {
		if ( false !== strpos( $image[0], 'images/media/default.png' ) ) {
			$image_url = wp_get_attachment_url( $attachment_id );
			$image[0]  = $image_url;
			$image[1]  = 60;
			$image[2]  = 60;
		}

		return $image;
	}

	public function maybe_update_image_url( $url, $attachment_id ) {
		if ( ! is_admin() && ! defined( 'REST_REQUEST' ) ) {
			return $url;
		}

		$attachment = get_post( $attachment_id );

		// If these all match it is likely an auto uploaded image via Fetch copy / paste.
		if ( $attachment->post_content === $attachment->post_title && $attachment->post_content === $attachment->guid ) {
			$url = $attachment->guid;
		}

		return $url;
	}

	public function product_carousel_preview_styles() {
		if ( ! is_admin() ) {
			return;
		}
	}

	public function edit_med() {
		?>
        <div class="hidden">
            <button class="open-shop-frame button fetch-tinymce-button">
                <img style="width: 16px; height: 16px; padding: 0 6px 0 0; top: 3px; position: relative; left: 0; box-sizing: initial;" src="<?php echo esc_url( plugins_url( '/assets/images/fetch-logo.png', dirname( __FILE__ ) ) ); ?>">Add / Edit Products
            </button>

            <script type="text/template" id="tmpl-manual-products">
				<?php include_once $this->pluginPath . 'views/templates/product-additions-widget.php'; ?>
            </script>
            <script type="text/template" id="tmpl-fetch-sidebar">
                <div class="media-frame-menu">
                    <div class="media-menu">
                        <a href="#" class="media-menu-item js-toggle-shop-button
					<# if (data.activeTab === 'shop-button') { #>
					active
					<# } #>">Shop Button</a>
                        <a href="#" class="media-menu-item js-toggle-product-carousel
					<# if (data.activeTab === 'product-carousel') { #>
					active
					<# } #>">Product Carousel</a>
                        <a href="#" class="media-menu-item js-toggle-manual-products
					<# if (data.activeTab === 'manual-products') {#>
					active
					<# } #>">Search for Products</a>
                    </div>
                </div>
            </script>
        </div>
		<?php
	}

	/**
	 * @return bool Whether or not this plugin has been loaded.
	 */
	public function load() {
		if ( $this->isLoaded() ) {
			return false;
		}

		$this->loadHelpers();

		foreach ( $this->getSubscribers() as $subscriber ) {
			$this->pluginApiManager->register( $subscriber );
		}

		$fetchShortcode = new FetchShortcode();
		$fetchShortcode->init();

		$fetchGutenberg = new FetchGutenberg();
		$fetchGutenberg->init();

		$this->loaded = true;

		return $this->isLoaded();
	}

	/**
	 * Add query var for previewing
	 *
	 * @param array $qvars Query vars.
	 *
	 * @return array Query vars.
	 */
	public function query_vars( $qvars ) {
		$qvars[] = 'preview-token';

		return $qvars;
	}

	/**
	 * If we're in the right context and have the preview query var set, run the preview logic.
	 *
	 * @param object $query WP_Query
	 *
	 * @return object WP_Query
	 */
	public function frenzy_preview( $query ) {
		if ( $query->get( 'preview-token' ) && $query->is_preview() && $query->is_main_query() && $query->is_singular() ) {
			if ( ! headers_sent() ) {
				nocache_headers();
			}
			add_filter( 'posts_results', [ $this, 'make_preview_available' ], 10, 2 );
		}

		return $query;
	}

	/**
	 * Make visible if the preview is valid
	 *
	 * @param array $posts The post to preview.
	 *
	 * @return array The post that is being previewed.
	 */
	public function make_preview_available( $posts ) {
		remove_filter( 'posts_results', [ $this, 'make_preview_available' ], 10 );

		if ( empty( $posts ) ) {
			return;
		}

		$post_id = $posts[0]->ID;

		if ( $this->can_preview( $post_id ) ) {
			$posts[0]->post_status = 'publish';
		}

		return $posts;
	}

	/**
	 * Check if preview is valid
	 *
	 * @param int $post_id Post id
	 *
	 * @return bool True if preview is valid
	 */
	private function can_preview( $post_id ) {
		if ( empty( $post_id ) ) {
			return false;
		}

		$api_key = get_option( 'fetch-account', [] );
		if ( $api_key ) {
			$api_key = $api_key['apiKey'];
		}

		if ( get_query_var( 'preview-token' ) !== $api_key ) {
			return false;
		}

		return true;
	}

	/**
	 * Checks if the plugin is loaded.
	 *
	 * @return bool Whether or not this plugin has been loaded.
	 */
	public function isLoaded() {
		return $this->loaded;
	}

	/**
	 * Registers all of the subscribers that power the plugin.
	 *
	 * @return array The collection of subscribers all of which make this plugin work.
	 */
	public function getSubscribers() {
		$frenzy_username = get_option( 'frenzy-username', '' );

		$fetch_api_key = HelperFunctions::get_fetch_api_key();

		return [
			new FetchTaxonomyFetchProduct(),
			new ScriptAssetSubscriber( $this->pluginUrl ),
			new PublicScriptAssetSubscriber( $this->pluginUrl ),
			new PublicStyleAssetSubscriber( $this->pluginUrl ),
			new AdminStyleAssetSubscriber( $this->pluginUrl ),
			new ShopWidgetSubscriber( $this->pluginPath ),
			new ToolsMenuSubscriber( $this->pluginPath ),
			new SaveSettingsSubscriber( $this->pluginPath ),
			new SavePaypalSubscriber( $this->pluginPath ),
			new AjaxSubscriber(
				$this->getApiClient(
					'FrenzyClient',
					$frenzy_username,
					$fetch_api_key
				),
				$this->cache
			),
			new ShopStyleAjaxSubscriber(
				$this->getApiClient(
					'ShopStyleClient',
					null,
					$fetch_api_key
				),
				$this->cache
			),
		];
	}

	/**
	 * Get the API client.
	 *
	 * @param string $class_name Name of the client class.
	 * @param string $username   The API username.
	 * @param string $api_key    The API key.
	 *
	 * @return mixed $client Retrieves the API client if it's not been set.
	 */
	private function getApiClient( $class_name, $username, $api_key ) {
		$client = null;

		if ( ! empty( $class_name ) ) {
			$class_name = "Frenzy\\API\\$class_name";
			$client     = new $class_name();
			if ( $client instanceof FrenzyClient ) {
				$client->setUsername( $username );
			}
			$client->setApiKey( $api_key );
			if ( $client instanceof ShopStyleClient ) {
				$client->set_api_endpoints();
			}
		}

		return $client;
	}

	private function loadHelpers() {
		require_once trailingslashit( $this->pluginPath ) . 'src/Helpers/helpers.php';
	}

	public function wp_ajax_fetch_get_preview_data() {
		$ids            = explode( ',', str_replace( ' ', '', sanitize_text_field( $_REQUEST['ids'] ) ) );
		$products       = get_terms(
			[
				'taxonomy'   => [
					'fetch-product',
				],
				'include'    => $ids,
				'hide_empty' => false,
				'orderby'    => 'include',
			]
		);
		$fetch_products = [];

		foreach ( $products as $product ) {
			$product_meta     = get_term_meta( $product->term_id, 'fetch_product_data', true );
			$fetch_products[] = [
				'image' => $product_meta['ImageUrls'][0],
				'brand' => $product_meta['Brand'],
				'title' => $product_meta['Title'],
				'price' => $product_meta['Price'],
			];
		}

		wp_send_json( $fetch_products );
	}

	public function frenzy_me_ajax_query_attachments() {
		if ( ! isset( $_REQUEST['query']['frenzyproducts'] ) || 'frenzyproducts' !== $_REQUEST['query']['frenzyproducts'] ) {
			return;
		}

		if ( ! current_user_can( 'upload_files' ) ) {
			wp_send_json_error();
		}

		$post_id       = absint( $_REQUEST['post_id'] );
		$products      = FetchTaxonomyFetchProduct::get_products_attached_to_post( $post_id );
		$product_terms = [];

		foreach ( $products as $manual_id => $term ) {
			if (
				! empty( $_REQUEST['query']['post__in'] ) &&
				false === in_array( $term->term_id, $_REQUEST['query']['post__in'] )
			) {
				continue;
			}

			$product_terms[] = get_term( $term );
		}

		$posts = array_map( [ $this, 'frenzy_prepare_term_for_js' ], $product_terms );
		$posts = array_filter( $posts );
		wp_send_json_success( $posts );
	}

	public function frenzy_ajax_modify_attachments( $posts ) {
		if ( ! isset( $_REQUEST['query']['frenzy'] ) || 'frenzy' !== $_REQUEST['query']['frenzy'] ) {
			return;
		}

		if ( ! current_user_can( 'upload_files' ) ) {
			wp_send_json_error();
			return;
		}

		$post_id = absint( $_REQUEST['post_id'] );
		$images  = FetchTaxonomyFetchProduct::get_images_for_post( $post_id );
		$counter = 'a';
		$response = [];
		foreach ( $images as $src => $image_id ) {
			$mime       = FetchTaxonomyFetchProduct::fetch_get_mime_type( $src );
			$response[] = array(
				'id'              => $image_id,
				'title'           => $counter,
				'filename'        => $src,
				'url'             => $src,
				'link'            => $src,
				'alt'             => '',
				'author'          => '',
				'description'     => '',
				'caption'         => '',
				'name'            => $src,
				'status'          => 'inherit',
				'uploadedTo'      => $post_id,
				'date'            => time(),
				'modified'        => time(),
				'menuOrder'       => 0,
				'mime'            => $mime,
				'type'            => 'image',
				'subtype'         => 'jpeg',
				'icon'            => $src,
				'dateFormatted'   => 'November 4, 2018',
				'nonces'          =>
					array(
						'update' => '3da17f13e1',
						'delete' => 'a9b0e651cb',
						'edit'   => 'cb4524fbb7',
					),
				'editLink'        => $src,
				'meta'            => false,
				'authorName'      => '',
				'uploadedToLink'  => $src,
				'uploadedToTitle' => $src,
				'context'         => '',
				'height'          => '',
				'width'           => '',
				'orientation'     => 'portrait',
				'sizes'           => [],
				'compat'          => [],
				'products'        => FetchTaxonomyFetchProduct::get_products_for_image( $image_id ),
			);

			$counter++;
		}

		$response = array_reverse( $response );
		wp_send_json_success( $response );
		exit;
	}

	public function frenzy_get_product_info() {
		$image_ids = FetchTaxonomyFetchProduct::get_images_for_post( absint( $_REQUEST['post_id'] ) );
		$img       = [];

		$available_in_product_carousel = false;
		foreach ( $image_ids as $image_id ) {
			$checked = '';
			if ( has_term( $_REQUEST['product_id'], 'fetch-product', $image_id ) ) {
				$checked                       = 'checked';
				$available_in_product_carousel = 'checked';
			}

			$url = get_post_field( 'guid', $image_id );

			$img[] = [
				'id'      => $image_id,
				'url'     => $url,
				'checked' => $checked,
			];
		}

		if ( false === $available_in_product_carousel && has_term( $_REQUEST['product_id'], 'fetch-product', $_REQUEST['post_id'] ) ) {
			$available_in_product_carousel = 'checked';
		}

		$response = [
			'id'               => absint( $_REQUEST['product_id'] ),
			'imgs'             => $img,
			'product_carousel' => $available_in_product_carousel,
		];
		wp_send_json_success( $response );
	}

	public function frenzy_prepare_term_for_js( $term_object ) {
		$term_meta = get_term_meta( $term_object->term_id, 'fetch_product_data', true );
		$type      = 'image';
		$subtype   = 'none';

		$images = FetchTaxonomyFetchProduct::get_images_for_post( absint( $_REQUEST['post_id'] ) );

		$img = [];
		foreach ( $images as $src => $image_id ) {
			$checked = '';
			if ( has_term( $term_object->term_id, 'fetch-product', $image_id ) ) {
				$checked = 'checked';
			}
			$img[] = [
				'id'      => $image_id,
				'url'     => $src,
				'checked' => $checked,
			];
		}

		$response = array(
			'allowLocalEdits' => true,
			'id'              => $term_object->term_id,
			'title'           => $term_object->name,
			'filename'        => $term_meta['Brand'],
			'url'             => $term_meta['ImageUrls'][0],
			'link'            => $term_meta['Source'],
			'alt'             => '',
			'author'          => '',
			'description'     => $term_object->description,
			'caption'         => $term_object->description,
			'name'            => $term_object->name,
			'status'          => 'publish',
			'uploadedTo'      => '',
			'date'            => '',
			'modified'        => '',
			'menuOrder'       => '',
			'mime'            => 'image/jpeg',
			'type'            => $type,
			'subtype'         => $subtype,
			'icon'            => $term_meta['ImageUrls'][0], // wp_mime_type_icon( $attachment_id ),
			'dateFormatted'   => '',
			'nonces'          => array(
				'update' => false,
				'delete' => false,
				'edit'   => false,
			),
			'editLink'        => false,
			// 'can' => [ 'save' => true ],
			'is_product'      => true,
			'price'           => $term_meta['Price'],
			'brand'           => $term_meta['Brand'],
			'source'          => $term_meta['Source'],
			'retailer'        => $term_meta['Retailer'],
			'imgs'            => $img,
			'user_commission' => $term_meta['user_commission'],
		);

		return $response;
	}


	public function frenzy_add_attachment_products( $response, $attachment, $meta ) {
		if ( ! isset( $_REQUEST['query']['frenzy'] ) || 'frenzy' !== $_REQUEST['query']['frenzy'] ) {
			// Make sure we fill a size out for images that don't have the size included.
			// This addresses an issue where Fetch saved images don't properly show in the media library.
			if ( ! isset( $response['sizes'] ) ) {
				$response['sizes'] = [
					'full' => [
						'orientation' => 'portrait',
						'url'         => $response['url'],
						'width'       => 1024,
						'height'      => 1024,
					],
				];
			}

			return $response;
		}

		$response['products'] = FetchTaxonomyFetchProduct::get_products_for_image( $response['id'] );
		return $response;
	}

	public function wpse_print_media_templates() {
		?>
        <script>
            jQuery( document ).ready( function ( $ ) {
                jQuery( "script#tmpl-attachment-details:first" ).remove();
                jQuery( "script#tmpl-attachment:first" ).remove();
            } );
        </script>
        <script type="text/html" id="tmpl-product-carousel-settings">
            <h2><?php esc_html_e( 'Choose Your Display Layout' ); ?></h2>

            <label class="setting">
                <span><?php esc_html_e( 'Layout' ); ?></span>
                <select class="columns" name="columns"
                        data-setting="columns">
                    <option value="carousel">
                        Carousel
                    </option>
                    <option value="columns-2">
                        2 Column Grid
                    </option>
                    <option value="columns-3">
                        3 Column Grid
                    </option>
                    <option value="columns-4">
                        4 Column Grid
                    </option>
                </select>
            </label>

        </script>
        <script type="text/html" id="tmpl-attachment-details">
            <h2>
                <# if ( data.is_product ) { #>
				<?php esc_html_e( 'Fetch Product Details' ); ?>
                <# } else { #>
				<?php esc_html_e( 'Attachment Details' ); ?>
                <# } #>
                <span class="settings-save-status">
					<span class="spinner"></span>
					<span class="saved"><?php esc_html_e( 'Saved.' ); ?></span>
				</span>
            </h2>
            <div class="attachment-info">
                <div class="thumbnail thumbnail-{{ data.type }}">
                    <# if ( data.uploading ) { #>
                    <div class="media-progress-bar">
                        <div></div>
                    </div>
                    <# } else if ( 'image' === data.type && data.sizes ) { #>
                    <img src="{{ data.size.url }}" draggable="false" alt=""/>
                    <# } else { #>
                    <img src="{{ data.icon }}" class="icon" draggable="false" alt=""/>
                    <# } #>
                </div>
                <div class="details">
                    <# if ( data.is_product ) { #>
                    <# if ( data.title ) { #>
                    <div class="filename">{{ data.title }}</div>
                    <# } #>
                    <# if ( data.price ) { #>
                    <div class="uploaded">Price: ${{ data.price }}</div>
                    <# } #>
                    <# if ( data.brand) { #>
                    <div class="uploaded">Brand: {{ data.brand }}</div>
                    <# } #>
                    <# if ( data.retailer) { #>
                    <div class="uploaded">Retailer: {{ data.retailer }}</div>
                    <# } #>
                    <# if ( data.source) { #>
                    <div class="uploaded">
                        <a href="{{ data.source }}" target="_blank">View Product Page</a>
                    </div>
                    <# } #>
                    <# } else { #>

                    <div class="filename">{{ data.filename }}</div>

                    <div class="uploaded">{{ data.dateFormatted }}</div>

                    <div class="file-size">{{ data.filesizeHumanReadable }}</div>
                    <# } #>
                    <# if ( 'image' === data.type && ! data.uploading && ! data.is_product ) { #>
                    <# if ( data.width && data.height ) { #>
                    <div class="dimensions">
						<?php
						/* translators: 1: a number of pixels wide, 2: a number of pixels tall. */
						printf( __( '%1$s by %2$s pixels' ), '{{ data.width }}', '{{ data.height }}' );
						?>
                    </div>
                    <# } #>

                    <# if ( data.can.save && data.sizes ) { #>
                    <a class="edit-attachment" href="{{ data.editLink }}&amp;image-editor" target="_blank"><?php _e( 'Edit Image' ); ?></a>
                    <# } #>
                    <# } #>

                    <# if ( data.fileLength && data.fileLengthHumanReadable ) { #>
                    <div class="file-length"><?php _e( 'Length:' ); ?>
                        <span aria-hidden="true">{{ data.fileLength }}</span>
                        <span class="screen-reader-text">{{ data.fileLengthHumanReadable }}</span>
                    </div>
                    <# } #>

                    <# if ( ! data.uploading && data.can.remove ) { #>
					<?php if ( MEDIA_TRASH ) : ?>
                        <# if ( 'trash' === data.status ) { #>
                        <button type="button" class="button-link untrash-attachment"><?php esc_html_e( 'Untrash' ); ?></button>
                        <# } else { #>
                        <button type="button" class="button-link trash-attachment"><?php esc_html_e( 'Trash' ); ?></button>
                        <# } #>
					<?php else : ?>
                        <button type="button" class="button-link delete-attachment"><?php esc_html_e( 'Delete Permanently' ); ?></button>
					<?php endif; ?>
                    <# } #>

                    <div class="compat-meta">
                        <# if ( data.compat && data.compat.meta ) { #>
                        {{{ data.compat.meta }}}
                        <# } #>
                    </div>
                </div>
            </div>
            <# if ( data.is_product ) {  #>
            <# if ( data.user_commission) { #>
            <div class="commission-amount">
                Commission Amount: ${{ data.user_commission }}
            </div>
            <# } #>
            <div class="filename"><b>Description</b></div>
            <div class="uploaded">{{ data.description }}</div>

            <h2 class="fetch-show-on-label">Add To
                <div class="auto-selection-options">
                    <button class="js-check-all">Check all</button>
                    <button class="js-uncheck-all">Uncheck All</button>
                </div>
                <div class="fetch-help-tooltip fetch-add-to-tooltip">
                    <div class="fetch-tooltip-icon">?</div>
                    <div class="fetch-help-tooltip-content">
                        <div class="fetch-tooltip-arrow-up"></div>
                        <p>- Choose which Shop Button images you want this product to be added to.</p></div>
                </div>
            </h2>
            <div class="frenzy-show-on-image-holder"></div>
            <# } #>
            <# if ( ! data.is_product ) { #>
            <label class="setting" data-setting="url">
                <span class="name"><?php _e( 'URL' ); ?></span>
                <input type="text" value="{{ data.url }}" readonly/>
            </label>
            <# var maybeReadOnly = data.can.save || data.allowLocalEdits ? '' : 'readonly'; #>
			<?php if ( post_type_supports( 'attachment', 'title' ) ) : ?>
                <label class="setting" data-setting="title">
                    <span class="name"><?php _e( 'Title' ); ?></span>
                    <input type="text" value="{{ data.title }}" {{ maybeReadOnly }}/>
                </label>
			<?php endif; ?>
            <# if ( 'audio' === data.type ) { #>
			<?php
			foreach (
				array(
					'artist' => __( 'Artist' ),
					'album'  => __( 'Album' ),
				) as $key => $label
			) :
				?>
                <label class="setting" data-setting="<?php echo esc_attr( $key ); ?>">
                    <span class="name"><?php echo $label; ?></span>
                    <input type="text" value="{{ data.<?php echo $key; ?> || data.meta.<?php echo $key; ?> || '' }}"/>
                </label>
			<?php endforeach; ?>
            <# } #>
            <label class="setting" data-setting="caption">
                <span class="name"><?php _e( 'Caption' ); ?></span>
                <textarea {{ maybeReadOnly }}>{{ data.caption }}</textarea>
            </label>
            <# if ( 'image' === data.type ) { #>
            <label class="setting" data-setting="alt">
                <span class="name"><?php _e( 'Alt Text' ); ?></span>
                <input type="text" value="{{ data.alt }}" {{ maybeReadOnly }}/>
            </label>
            <# } #>
            <label class="setting" data-setting="description">
                <span class="name"><?php _e( 'Description' ); ?></span>
                <textarea {{ maybeReadOnly }}>{{ data.description }}</textarea>
            </label>
            <# } else { #>

            <# } #>
        </script>
        <script type="text/html" id="tmpl-attachment">
            <style>
                .wp-core-ui .attachment .filename .product-title {
                    padding: 3px 3px 0;
                    font-size: 11px;
                    text-overflow: unset;
                    white-space: nowrap;
                }

                .wp-core-ui .attachment .filename .product-price {
                    padding: 0 0 2px;
                }
            </style>
            <div class="attachment-preview js--select-attachment type-{{ data.type }} subtype-{{ data.subtype }} {{ data.orientation }}">
                <div class="thumbnail">
                    <# if ( data.uploading ) { #>
                    <div class="media-progress-bar">
                        <div style="width: {{ data.percent }}%"></div>
                    </div>
                    <# } else if ( 'image' === data.type && data.sizes && ! data.is_product ) { #>
                    <div class="centered">
                        <img src="{{ data.size.url }}" draggable="false" alt=""/>
                    </div>
                    <# } else if ( data.is_product ) { #>
                    <div class="centered">
                        <# if ( data.image && data.image.src && data.image.src !== data.icon ) { #>
                        <img src="{{ data.image.src }}" class="thumbnail" draggable="false" alt=""/>
                        <# } else if ( data.sizes && data.sizes.medium ) { #>
                        <img src="{{ data.sizes.medium.url }}" class="thumbnail" draggable="false" alt=""/>
                        <# } else { #>
                        <img src="{{ data.icon }}" class="icon" draggable="false" alt=""/>
                        <# } #>
                    </div>
                    <div class="filename">
                        <div class="product-title">{{ data.title }}</div>
                        <div class="product-price">${{ data.price }}</div>
                    </div>
                    <# } else { #>
                    <div class="centered">
                        <# if ( data.image && data.image.src && data.image.src !== data.icon ) { #>
                        <img src="{{ data.image.src }}" class="thumbnail" draggable="false" alt=""/>
                        <# } else if ( data.sizes && data.sizes.medium ) { #>
                        <img src="{{ data.sizes.medium.url }}" class="thumbnail" draggable="false" alt=""/>
                        <# } else { #>
                        <img src="{{ data.icon }}" class="icon" draggable="false" alt=""/>
                        <# } #>
                    </div>
                    <div class="filename">
                        <div>{{ data.filename }}</div>
                    </div>
                    <# } #>
                </div>
                <# if ( data.buttons.close ) { #>
                <button type="button" class="button-link attachment-close media-modal-icon">
                    <span class="screen-reader-text"><?php esc_html_e( 'Remove' ); ?></span></button>
                <# } #>
            </div>
            <# if ( data.type == 'image' && ! data.is_product && data.products ) { #>
            <div class="attachment-products">
                <# for( var product in data.products) { #>
                <div style="display:inline-block; position:relative;" class="attachment-product" data-term-id="{{data.products[product].id}}" data-term-url="{{data.products[product].ImageUrls[0]}}" data-term-title="{{data.products[product].Title}}" data-term-price="{{data.products[product].Price}}" data-term-brand="{{data.products[product].Brand}}" data-term-description="{{data.products[product].description}}" data-term-source="{{data.products[product].Source}}" data-term-retailer="{{data.products[product].Retailer}}" data-user_commission="{{data.products[product].user_commission}}">
                    <div class="product-image-container">
                        <img src="{{data.products[product].ImageUrls[0]}}">
                    </div>
                    <div class="product-container">
                        <div>{{data.products[product].Brand}}</div>
                        <div>{{data.products[product].Price}}</div>
                    </div>
                    <button type="button" class="button-product-remove">
                        <span class="screen-reader-text"><?php esc_html_e( 'Remove' ); ?></span></button>
                </div>
                <#
                } #>
            </div>
            <# } #>
            <# if ( data.buttons.check ) { #>
            <button type="button" class="check" tabindex="-1">
                <span class="media-modal-icon"></span><span class="screen-reader-text"><?php esc_html_e( 'Deselect' ); ?></span>
            </button>
            <# } #>
            <#
            var maybeReadOnly = data.can.save || data.allowLocalEdits ? '' : 'readonly';
            if ( data.describe ) {
            if ( 'image' === data.type ) { #>
            <input type="text" value="{{ data.caption }}" class="describe" data-setting="caption"
                   placeholder="<?php esc_attr_e( 'Caption this image&hellip;' ); ?>" {{ maybeReadOnly }}/>
            <# } else { #>
            <input type="text" value="{{ data.title }}" class="describe" data-setting="title"
            <# if ( 'video' === data.type ) { #>
            placeholder="<?php esc_attr_e( 'Describe this video&hellip;' ); ?>"
            <# } else if ( 'audio' === data.type ) { #>
            placeholder="<?php esc_attr_e( 'Describe this audio file&hellip;' ); ?>"
            <# } else { #>
            placeholder="<?php esc_attr_e( 'Describe this media file&hellip;' ); ?>"
            <# } #> {{ maybeReadOnly }} />
            <# }
            } #>
        </script>
        <script type="text/html" id="tmpl-editor-fetch-banner">
            <div class="editor" id="">
                <ul class="products frenzy-product-carousel {{ data.type }}">
                    <# for( var product in data.products) { #>
                    <li class="product">
                        <div class="product-inner">
                            <div class="frenzy-single-image">
                                <img src="{{ data.products[product].image }}" class="frenzy-clothing">
                            </div>
                            <div class="frenzy-brand">{{ data.products[product].brand }}</div>
                            <div class="frenzy-item-title">{{ data.products[product].title }}</div>
                            <div class="frenzy-price">${{ data.products[product].price }}</div>
                        </div>
                    </li>
                    <# } #>
                </ul>
            </div>
        </script>
		<?php
	}

	public function frenzy_me_ajax_save_image_products() {
		// See if we need to save a new product.
		if ( ! empty( $_POST['productData'] ) ) {
			$product                          = new \stdClass();
			$product->details                 = new \stdClass();
			$product->retailer                = new \stdClass();
			$product->price                   = sanitize_text_field( $_POST['productData']['price'] );
			$product->imageUrl                = esc_url_raw( $_POST['productData']['image_url'] );
			$product->clickUrl                = esc_url_raw( $_POST['productData']['buy_url'] );
			$product->name                    = sanitize_text_field( $_POST['productData']['title'] );
			$product->details->brand          = sanitize_text_field( $_POST['productData']['manufacturer'] );
			$product->retailer->name          = sanitize_text_field( $_POST['productData']['merchant_name'] );
			$product->details->in_stock       = 1;
			$product->details->sale           = 0;
			$product->description             = sanitize_text_field( $_POST['productData']['description'] );
			$product->score                   = 0;
			$product->id                      = sanitize_text_field( $_POST['productData']['sku'] );
			$product->brand_category_group_id = ( ! empty( $_POST['productData']['item_group_id'] ) ) ? sanitize_text_field( $_POST['productData']['item_group_id'] ) : '';
			$product->inStock                 = 1;
			$product->is_manual               = true;
			$product->details->commission     = sanitize_text_field( $_POST['productData']['user_commission'] );

			// Set the global product to the term ID that's created so it's compatible.
			$_POST['product'] = FetchTaxonomyFetchProduct::create( $product );
		}

		if ( ! empty( $_POST['images'] ) ) {
			foreach ( $_POST['images'] as $attachment_id => $value ) {
				if ( 'true' == $value ) {
					wp_set_post_terms( absint( $attachment_id ), [ absint( $_POST['product'] ) ], 'fetch-product', true );
				} else {
					wp_remove_object_terms( absint( $attachment_id ), [ absint( $_POST['product'] ) ], 'fetch-product' );
				}
			}
		}

		// If the product carousel checkbox is checked make sure we attach it to the post.
		if ( ! empty( $_POST['product_carousel'] ) ) {
			wp_set_post_terms( $_POST['product_carousel'], [ absint( $_POST['product'] ) ], 'fetch-product', true );
		}

		wp_send_json_success();
	}

	public function frenzy_ajax_save_sort_order() {
		$post_id = absint( $_POST['postId'] );
		$order   = array_map( 'absint', $_POST['productOrder'] );
		if ( empty( $post_id ) || empty( $order ) ) {
			wp_send_json_success();
		}

		$update = wp_set_object_terms( $post_id, $order, 'fetch-product', false );
		wp_send_json_success( $update );
	}

	public function frenzy_remove_product_from_image() {
		if ( empty( $_POST['img_id'] ) || empty( $_POST['product_id'] ) ) {
			return;
		}

		$success = wp_remove_object_terms( absint( $_POST['img_id'] ), [ absint( $_POST['product_id'] ) ], 'fetch-product' );
		wp_send_json_success( $success );
	}

	public function fetch_save_draft_post_content() {
		$post_id            = absint( $_REQUEST['post_id'] );
		$post               = get_post( $post_id );
		$post_thumbnail     = absint( $_REQUEST['post_thumbnail'] );
		$post->post_content = wp_unslash( $_REQUEST['post_content'] );
		$post->post_title   = sanitize_post_field( 'post_title', $_REQUEST['post_title'], $post_id );

		wp_update_post( $post );
		set_post_thumbnail( $post, $post_thumbnail );
		$images = FetchTaxonomyFetchProduct::get_images_for_post( $post_id );
		wp_send_json_success( [ 'post_images' => $images ] );
	}

	/**
	 * Call to save all fetch post settings.
	 */
	public function ajax_fetch_save_shop_settings() {
		$post_id       = absint( $_POST['post_id'] );
		$price_status = absint( $_POST['show_price_val'] );
		$button_status = absint( $_POST['shop_button_val'] );

		$updated_post_settings = array(
			'showShopButton' => $button_status,
			'showPrice'  => $price_status
		);

		$update = update_post_meta( $post_id, 'fetch_post_settings', $updated_post_settings );
		wp_send_json_success( $update );
	}
}
