<?php
namespace Frenzy\PluginAPI;

use Frenzy\Helpers\HelperFunctions;

/**
 * The abstract class responsible for defining the functionality that any Subscriber should
 * implement.
 *
 * @since 1.0.0
 */
abstract class AbstractSubscriber implements HookSubscriberInterface {

	/**
	 * @param string $pluginLocation The location of the plugin on disk.
	 */
	abstract public function __construct( $pluginLocation);

	/**
	 * {@inheritdoc}
	 */
	public static function getHooks() {
		return array();
	}

	/**
	 * Determines whether or not the API should be contacted.
	 *
	 * @access protected
	 */
	protected function shouldContactApi() {
		if ( ! $this->requiredSettingsConfigured() ) {
			return false;
		}

		if ( $this->isSingleAndEnabled() ) {
			return true;
		}

		// If we're on the homepage and the option to enable on home feed is enabled.
		global $wp_query;
		$frenzyShowHomeFeed = get_option( 'fetch-show-home-feed' );
		if (
			is_home()
			&& ! empty( $frenzyShowHomeFeed )
			&& isset( $wp_query->in_the_loop )
			&& $wp_query->in_the_loop
			&& HelperFunctions::frenzy_is_enabled( get_the_ID() )
		) {
			return true;
		}

		return false;
	}

	protected function shouldLoadAssets() {
		if ( ! $this->requiredSettingsConfigured() ) {
			return false;
		}

		if ( $this->isSingleAndEnabled() ) {
			return true;
		}

		if ( ismain ) {

			$frenzyShowHomeFeed = get_option( 'fetch-show-home-feed' );
		}
		if ( ! empty( $frenzyShowHomeFeed ) ) {
			return true;
		}

		return false;
	}

	protected function requiredSettingsConfigured() {
		$frenzyApiKey = get_option( 'fetch-account', [] );
		if ( $frenzyApiKey ) {
			$frenzyApiKey = $frenzyApiKey['apiKey'];
		}

		if ( empty( $frenzyApiKey ) ) {
			return false;
		}

		return true;
	}

	protected function isSingleAndEnabled() {
		if ( is_single() && HelperFunctions::frenzy_is_enabled( get_the_ID() ) ) {
			return true;
		}

		return false;
	}

	protected function isFrenzyEnabledOnPost( $postId ) {
		return ( 'on' === get_post_meta( $postId, 'frenzy-enabled', true ) );
	}
}
