<?php

namespace Frenzy\Subscriber;

use Frenzy\PluginAPI\AbstractSubscriber;
use Frenzy\PluginAPI\HookSubscriberInterface;

/**
 * Adds the administration stylesheets.
 *
 * @since 1.0.0
 */
class AdminStyleAssetSubscriber extends AbstractSubscriber implements HookSubscriberInterface {
	/**
	 * @var string $pluginUrl The URL of the on disk.
	 */
	private $pluginUrl;

	/**
	 * @var string $current_screen The current WP screen.
	 */
	private $current_screen;

	/**
	 * Initializes the class.
	 *
	 * @param string $pluginUrl A reference to the URL of this plugin.
	 */
	public function __construct( $pluginUrl ) {
		$this->pluginUrl = trailingslashit( $pluginUrl );
	}

	/**
	 * {@inheritdoc}
	 */
	public static function getHooks() {
		return [
			'admin_enqueue_scripts' => 'enqueue',
		];
	}

	/**
	 * If we're not a settings screen, then adds the admin stylesheet.
	 */
	public function enqueue() {
		$this->current_screen = get_current_screen();

		if ( ! $this->isSettingsScreen() && ! $this->isPostEditScreen() ) {
			return;
		}

		wp_enqueue_style(
			'frenzy-wp',
			$this->pluginUrl . 'assets/styles/css/admin.css',
			[],
			filemtime( dirname( dirname( dirname( __FILE__ ) ) ) . '/assets/styles/css/admin.css' )
		);

		if ( $this->isPostEditScreen() ) {
			wp_enqueue_style(
				'frenzy-products-chosen-10styles',
				$this->pluginUrl . 'assets/styles/css/chosen.min.css',
				[],
				filemtime( dirname( dirname( dirname( __FILE__ ) ) ) . '/assets/styles/css/chosen.min.css' )
			);
			wp_enqueue_style(
				'frenzy-products-pagination-styles',
				$this->pluginUrl . 'assets/styles/css/simple-pagination.css',
				[ 'frenzy-products-chosen-styles' ],
				filemtime( dirname( dirname( dirname( __FILE__ ) ) ) . '/assets/styles/css/simple-pagination.css' )
			);

			wp_enqueue_style(
				'frenzy-carousel-preview',
				$this->pluginUrl . 'assets/styles/css/carousel-preview.css',
				[],
				time()
			);

			add_editor_style( $this->pluginUrl . 'assets/styles/css/carousel-preview.css' );
		}
	}

	/**
	 * Determines if we're on a settings screen.
	 *
	 * @access private
	 *
	 * @return true if we're on the Frenzy Settings screen.
	 */
	private function isSettingsScreen() {
		return 'settings_page_fetch-settings-submenu' === $this->current_screen->id;
	}

	/**
	 * Determines if we're on a post edit screen.
	 *
	 * @access private
	 *
	 * @return true if we're on a post edit screen.
	 */
	private function isPostEditScreen() {
		if ( ! is_admin() ) {
			return false;
		}

		return ( 'post' === $this->current_screen->id ) && 'add' === $this->current_screen->action || ( isset( $_GET['action'] ) && ( 'edit' === $_GET['action'] ) );
	}
}
