<?php
namespace Frenzy\Subscriber;

use Frenzy\Inc\FetchTaxonomyFetchProduct;
use Frenzy\PluginAPI\AbstractSubscriber;
use Frenzy\PluginAPI\HookSubscriberInterface;
use Frenzy\Helpers\HelperFunctions;

/**
 * Adds the public JavaScript file to a single page.
 *
 * @since 1.0.0
 */
class PublicScriptAssetSubscriber extends AbstractSubscriber implements HookSubscriberInterface {

	/**
	 * @var string $pluginUrl The URL of the on disk.
	 */
	private $pluginUrl;

	/**
	 * Initializes the class.
	 *
	 * @param string $pluginUrl A reference to the URL of this plugin.
	 */
	public function __construct( $pluginUrl ) {
		$this->pluginUrl = $pluginUrl;
	}

	/**
	 * {@inheritdoc}
	 */
	public static function getHooks() {
		return array(
			'wp_enqueue_scripts' => 'enqueue',
		);
	}

	/**
	 * Determines if we should contact the API and, if so, adds the JavaScript for talking to the API and for building
	 * the carousel.
	 */
	public function enqueue() {
		// Note: Using the unminified version of the script for display because
		// the minified version currently causes problems with the 'Shop' button.
		wp_enqueue_script(
			'frenzy-wp',
			$this->pluginUrl . 'assets/scripts/dev/public.js',
			array( 'jquery' ),
			filemtime( dirname( dirname( dirname( __FILE__ ) ) ) . '/assets/scripts/dev/public.js' ),
			true
		);

		wp_localize_script(
			'frenzy-wp',
			'frenzy',
			array(
				'ajax_url'  => admin_url( 'admin-ajax.php' ),
				'security'  => wp_create_nonce( 'frenzy-security-nonce' ),
				'posts'     => [],
				'flags'     => [
					'eventsAdded'            => false,
					'shopSimilarEventsAdded' => [],
				],
				'carousels' => $this->carousel_output(),
			)
		);

		wp_localize_script( 'frenzy-wp', 'frenzyPostSettings', HelperFunctions::get_fetch_shop_settings( get_the_ID() ) );
	}

	/**
	 * Generates an array where key is image ID and the values are arrays of products associated with the image.
	 */
	public function carousel_output() {
		global $wp_query;
		$posts = [];

		foreach ( $wp_query->posts as $post ) {
			global $post;
			$prods  = [];
			$images = FetchTaxonomyFetchProduct::get_images_for_post( $post->ID );
			foreach ( $images as $source => $image ) {
				$products = FetchTaxonomyFetchProduct::get_products_for_image( $image );
				if ( empty( $products ) ) {
					continue;
				}

				$prods[ $source ] = $products;
			}

			if ( empty( $prods ) ) {
				continue;
			}

			$posts[ $post->ID ] = $prods;
		}

		return $posts;
	}

}
