<?php
namespace Frenzy\Subscriber;

use Frenzy\Helpers\HelperFunctions;
use Frenzy\PluginAPI\AbstractSubscriber;
use Frenzy\PluginAPI\HookSubscriberInterface;

/**
 * Adds the public stylesheets file to a single page.
 *
 * @since 1.0.0
 */
class PublicStyleAssetSubscriber extends AbstractSubscriber implements HookSubscriberInterface {

	/**
	 * @var string $pluginUrl The URL of the on disk.
	 */
	private $pluginUrl;

	/**
	 * Initializes the class.
	 *
	 * @param string $pluginUrl A reference to the URL of this plugin.
	 */
	public function __construct( $pluginUrl ) {
		$this->pluginUrl = $pluginUrl;
	}

	/**
	 * {@inheritdoc}
	 */
	public static function getHooks() {
		 return array(
			 'wp_enqueue_scripts' => 'enqueue',
			 'body_class'         => 'body_classes',
		 );
	}

	/**
	 * Determines if we should contact the API and adds the CSS for the single page.
	 */
	public function enqueue() {
		wp_enqueue_style(
			'frenzy-wp',
			$this->pluginUrl . 'assets/styles/css/public.css',
			array(),
			filemtime( dirname( dirname( dirname( __FILE__ ) ) ) . '/assets/styles/css/public.css' ),
			'all'
		);
	}

	/**
	 * Filter body classes for styling, etc.
	 *
	 * @param array $classes The current body classes.
	 *
	 * @return array
	 */
	public function body_classes( $classes ) {
		if ( wp_is_mobile() ) {
			array_push( $classes, 'mobile' );
		}

		return $classes;
	}
}
