<?php

namespace Frenzy\Subscriber;

use Frenzy\API\Indi;
use Frenzy\Helpers\HelperFunctions;
use Frenzy\PluginAPI\AbstractSubscriber;
use Frenzy\PluginAPI\HookSubscriberInterface;

/**
 * Adds the Frenzy submenu to the WordPress 'Tools' menu.
 */
class ToolsMenuSubscriber extends AbstractSubscriber implements HookSubscriberInterface {
	/**
	 * @var string The path to the plugin on disk.
	 */
	private $pluginPath;

	/**
	 * Initializes the plugin by setting it's plugin path.
	 *
	 * @param string $pluginPath A reference to the path of the plugin on disk.
	 */
	public function __construct( $pluginPath ) {
		$this->pluginPath = $pluginPath;
	}

	/**
	 * @return array The collection of hooks to which this subscriber is attached.
	 */
	public static function getHooks() {
		return array(
			'admin_menu'     => 'addSubmenuPage',
			'init'           => 'maybeFetchLogout',
			'admin_bar_menu' => [ 'addCommissionAdminBarIcon', 200 ],
		);

		// add_action( 'wp_enqueue_scripts', function()
		// {
		// wp_enqueue_style( /* register your stylesheet */ );
		// }
	}

	/**
	 * Adds a submenu page for the plugin's Settings.
	 */
	public function addSubmenuPage() {
		add_submenu_page(
			'options-general.php',
			'Fetch',
			'Fetch',
			'manage_options',
			'fetch-settings-submenu',
			array( $this, 'display' )
		);
	}

	/**
	 * Renders the settings options for the plugin page.
	 */
	public function display() {
		$apiKey       = HelperFunctions::get_fetch_api_key();
		$showHomeFeed = get_option( 'fetch-show-home-feed', 0 );
		include_once $this->pluginPath . 'views/settings.php';
	}

	public function maybeFetchLogout() {
		if ( ! isset( $_GET['fetch-logout-account'] ) ) {
			return;
		}

		HelperFunctions::fetch_logout();
		wp_redirect( admin_url( 'options-general.php?page=fetch-settings-submenu&tab=accounts' ) );
		exit;
	}

	public function addCommissionAdminBarIcon( \WP_Admin_Bar $bar ) {
		$indi = new Indi();
		$bar->add_menu(
			array(
				'id'    => 'wpse',
				'title' => '<span class="ab-icon" style="width: 24px; height: 22px; background: url(' . FRENZY_ASSETS_URL . 'assets/images/commission-button-admin-bar.png) !important; background-repeat: no-repeat !important; background-position: center center !important;"></span>' . $indi->getPayable(),
				'href'  => admin_url( 'options-general.php?page=fetch-settings-submenu&tab=analytics' ),
				'meta'  => array(
					'target' => '_self',
					'title'  => __( '', 'fetch' ),
					'html'   => '<!-- Custom HTML that goes below the item -->',
				),
			)
		);
	}
}
