<?php

use Frenzy\Cache\FrenzyCache;
use Frenzy\Helpers\HelperFunctions;

global $post;
$cache          = new FrenzyCache();
$post_permalink = get_post_permalink( $post->ID );
// Get stored products from cache or the post meta, depending...
if ( $cache->has( $post->ID ) ) {
	$stored_product_data = $cache->get( $post->ID );
} else {
	$stored_product_data = get_post_meta( $post->ID, 'frenzy-response', true );
	if ( empty( $stored_product_data ) ) {
		$client              = new \Frenzy\API\FrenzyClient();
		$ajax_subscriber     = new \Frenzy\Subscriber\AjaxSubscriber( $client, $cache );
		$stored_product_data = $ajax_subscriber->get_added_shopstyle_products( $post->ID );
	}
}

if ( ! empty( $stored_product_data ) ) : ?>
	<form action="POST">
		<p class="shopsimilar-products-order-list-desc">Drag and drop to reorder items for view on the front-end of the site. Use the "X" to permanently remove a product from the list or the Add Additional Products button above to add products individually. Use the Refresh Product List button to refresh products from the Frenzy API.</p>
		<ul id="js-shopsimilar-products-list" class="shopsimilar-products-order-list">
			<li id="shopsimilar-products-order-list-item-template" class="shopsimilar-products-order-list-item ui-sortable-handle invisible" data-id="" data-oldindex="0" data-newindex="0" data-product-data-src="">
				<div class="shopstyle-products-details">
					<div>
						<img width="36" src="" alt="" scale="0">
					</div>
					<div class="shopsimilar-products-details-meta">
						<strong class="js-shopsimilar-products-product-name"></strong>
						<p></p>
					</div>
				</div>
				<a class="shopsimilar-products-remove" href="#"><span></span></a>
			</li>
			<?php foreach ( $stored_product_data as $key => $item ) : ?>
				<li class="shopsimilar-products-order-list-item" data-id="<?php echo ! empty( $item['ItemID'] ) ? esc_attr( $item['ItemID'] ) : ''; ?>" data-oldindex="<?php echo ! empty( $key ) ? esc_attr( $key ) : ''; ?>" data-newindex="<?php echo ! empty( $key ) ? esc_attr( $key ) : ''; ?>" data-product-data-src="<?php echo ! empty( $item['productDataSource'] ) ? esc_attr( $item['productDataSource'] ) : ''; ?>">
					<div class="shopstyle-products-details">
						<div>
							<img width="36" src="<?php echo ( ! empty( $item['ImageUrls'] ) && ! empty( $item['ImageUrls'][0] ) ) ? esc_url( HelperFunctions::clean_url( $item['ImageUrls'][0] ) ) : ''; ?>" alt="<?php echo ! empty( $item['Title'] ) ? esc_html( $item['Title'] ) : ''; ?>">
						</div>
						<div class="shopsimilar-products-details-meta">
							<strong class="js-shopsimilar-products-product-name"><?php echo ! empty( $item['Title'] ) ? esc_html( $item['Title'] ) : ''; ?></strong>
							<p><?php echo ! empty( $item['Price'] ) ? esc_html( $item['Price'] ) : ''; ?></p>
						</div>
					</div>
					<a class="shopsimilar-products-remove" href="#"><span></span></a>
				</li>
			<?php endforeach; ?>
		</ul>
	</form>
<?php else : ?>
	<p>There are currently no products to show. Use the "Update Products" button under "Frenzy for WordPress" to the right of this page to schedule an update, or
		<a href="<?php echo esc_url( $post_permalink ); ?>" target="_blank">visit this article</a>, then revisit this page to refresh products now.
	</p>
	<ul id="js-shopsimilar-products-list" class="shopsimilar-products-order-list">
		<li id="shopsimilar-products-order-list-item-template" class="shopsimilar-products-order-list-item ui-sortable-handle invisible" data-id="" data-oldindex="0" data-newindex="0" data-product-data-src="">
			<div class="shopstyle-products-details">
				<div>
					<img width="36" src="" alt="" scale="0">
				</div>
				<div class="shopsimilar-products-details-meta">
					<strong class="js-shopsimilar-products-product-name"></strong>
					<p></p>
				</div>
			</div>
			<a class="shopsimilar-products-remove" href="#"><span></span></a>
		</li>
		<?php // Populated with individual products via JS from the ShopSimilar API using the "Add Products" button in the post meta box UI. ?>
	</ul>
<?php endif; ?>
<?php wp_nonce_field( 'frenzy_shopsimilar_product_reordering', '_product_reordering_nonce' ); ?>
<?php add_thickbox(); ?>
<div class="lds-ring shopstyle-ajax-loader invisible"><div></div><div></div><div></div><div></div></div>
