<?php
require_once __DIR__ . '/../templates/paypal-message.php';
$indi                  = new \Frenzy\API\Indi();
$commissions           = $indi->getCommissionData();
$minimum_payout        = $indi->minimum_payout;
$paypal_email          = get_option( 'fetch-paypal-email', '' );
$get_payment_status    = get_option( 'fetch-check-payout-status' );
$get_payout_history    = ! empty( $indi->getPayoutHistory() ) ? $commissions->getPayoutHistory() : json_decode( json_encode ( get_option( 'fetch-payouts', [] ) ) );
$pending_payout_amount = 0;
//Get the the pending payout amount
if ( !  empty( $get_payout_history ) ) {
	$payout = end($get_payout_history);
	// Get the payout history from the Indi API (This endpoint does not actually exist yet but leave here as example)
	if (isset($payout->status) && 'pending' === $payout->status) {
		$pending_payout_amount = $payout->payout;
	} else if ($payout = wp_list_pluck($get_payout_history, 'payout')) {
		$pending_payout_amount = end($payout);

	}
}

?>
<h1 class="title">PayPal</h1>
<form method="post" action="<?php echo esc_html( admin_url( 'admin-post.php' ) ); ?>" class="fetch-settings">
	<table class="form-table">
		<tbody>
		<tr>
			<th scope="row">
				<label for="paypal-email">PayPal email</label>
			</th>
			<td class="form-row">
				<input id="paypal-email" type="email" name="fetch-paypal-email" autocorrect="off" autocapitalize="none" placeholder="example@example.com" value="<?php echo esc_attr( $paypal_email ); ?>">
			</td>
		</tr>
		</tbody>
	</table>
	<?php wp_nonce_field( 'fetch-wordpress-plugin-paypal-email', 'fetch-wordpress-plugin' ); ?>
	<?php submit_button( 'Save PayPal Email Address', 'primary', 'fetch-save-paypal-email-settings', false ); ?>
</form>

<h2>Request Payout</h2>
<?php if ( $commissions->PayableCommission >= $minimum_payout ) : ?>
	<form method="post" action="<?php echo esc_html( admin_url( 'admin-post.php' ) ); ?>">
		<p><?php printf( 'Request a payout when you earn <strong>$%s or more</strong> in earned commission. You currently have <strong>%s</strong> in earned commission.', number_format( $minimum_payout, 2 ), $indi->getPayable() ); ?></p>
		<?php if ( ! empty( $get_payment_status ) && ! empty( $pending_payout_amount ) ) : ?>
			<div class="notice notice-warning">
				<p><?php printf( 'You currently have a pending payout of %s. Requesting another payout while this payment is processing will not increase payout time.', $pending_payout_amount ); ?></p>
			</div>
		<?php endif; ?>
		<?php wp_nonce_field( 'fetch-wordpress-plugin-paypal-request-payout',
			'fetch-wordpress-plugin' ); ?>
		<?php submit_button( sprintf( 'Request a Payout of %s', $indi->getPayable() ), 'primary', 'fetch-save-paypal-payout-settings', false ); ?>
	</form>
<?php else: ?>
	<p>Earn at least $<?php echo number_format( $minimum_payout, 2 ); ?> to cash out through PayPal! You currently have <strong><?php echo $indi->getPayable(); ?></strong> in earned commission.</p>
<?php endif; ?>

<?php if ( ! empty( $get_payout_history ) ): ?>
	<h2>Payout History</h2>
	<table class="fetch-table payout-history widefat">
		<thead>
		<th class="fetch-table-heading row-title">Payout Amount</th>
		<th class="fetch-table-heading row-title">Date Requested</th>
		<th class="fetch-table-heading row-title">Payout Status</th>
		</thead>
		<tbody>
		<?php foreach ( $get_payout_history as $payout ): ?>
			<tr>
				<td class="payout-amount"><?php echo $payout->payout; ?></td>
				<td class="payout-request-date" data-timestamp="<?php echo esc_attr( $payout->time ); ?>"><?php echo $payout->time; ?></td>
				<td class="payout-request-status"><?php echo isset( $payout->request_status ) ? $payout->request_status : 'Processed' ?></td>
			</tr>
		<?php endforeach; ?>
		</tbody>
	</table>
<?php endif; ?>
